/*
 ************************************************************************************************
 *                                                                                              *
 *  OSA cooperative RTOS for Microchip PIC-controllers: PIC10/12/16/18/24/dsPIC                 *
 *                                                                                              *
 *  OSA is distributed under BSD license (see license.txt)                                      *
 *                                                                                              *
 *  URL:        http://wiki.pic24.ru/doku.php/en/osa/ref/intro                                  *
 *              http://picosa.narod.ru                                                          *
 *                                                                                              *
 *----------------------------------------------------------------------------------------------*
 *                                                                                              *
 *  File:       osa_smsg.h                                                                      *
 *                                                                                              *
 *  Programmer: Timofeev Victor                                                                 *
 *              osa@pic24.ru, testerplus@mail.ru                                                *
 *                                                                                              *
 *  Definition: Services for work with simple messages                                          *
 *                                                                                              *
 *  History:    21.01.2009                                                                      *
 *              06.04.2009 -    Service OS_IsSMsgFree declaration corrected.                    *
 *              18.06.2009 -    Argument "message" placed in braces                             *
 *                              Alternate names added                                           *
 *              29.10.2009 -    Type 'bit' replaced with __osa_bool for _OS_CheckSmsg           *
 *              04.11.2009 -    All OS_SMsg_xxx renamed to OS_Smsg_xxx                          *
 *              -                                                                               *
 *                                                                                              *
 ************************************************************************************************
 */





/************************************************************************************************
 *                                                                                              *
 *                              S I M P L E     M E S S A G E S                                 *
 *                                                                                              *
 ************************************************************************************************/


#if defined(OS_ENABLE_INT_SMSG) && (OS_SMSG_SIZE != 1)

    #ifndef _OS_temp_I_ENABLE
    #define _OS_temp_I_ENABLE
    #endif
    #define __OS_SMSG_DI()       {_OS_temp_I = OS_DI();}
    #define __OS_SMSG_RI()       {OS_RI(_OS_temp_I);}

#else

    #define __OS_SMSG_DI()
    #define __OS_SMSG_RI()

#endif


#if !defined(OS_ENABLE_INT_SMSG) || (OS_SMSG_SIZE == 1)
    // Check for message exists
    #define OS_Smsg_Check(smsg)       (smsg != (OST_SMSG)0)

    // Check for message is free
    #define OS_Smsg_IsFree(smsg)      (smsg == (OST_SMSG)0)
#else
    __osa_bool _OS_Smsg_Check(OST_SMSG*);

    // Check for message exists
    #define OS_Smsg_Check(smsg)       _OS_Smsg_Check(&(smsg))

    // Check for message is free
    #define OS_Smsg_IsFree(smsg)      (!_OS_Smsg_Check(&(smsg)))
#endif


//------------------------------------------------------------------------------
// Internal macro: Clear message (internal system service)
#define _OS_Smsg_MakeFree(smsg)        smsg = (OST_SMSG)0


//------------------------------------------------------------------------------
// Create simple message

#define OS_Smsg_Create(smsg)                                            \
    {                                                                   \
        __OS_SMSG_DI();                                                 \
        _OS_Smsg_MakeFree(smsg);                                        \
        __OS_SMSG_RI();                                                 \
    }                                                                   \


//------------------------------------------------------------------------------
// Send simple message. If message already exists then wait when it became free.

#define OS_Smsg_Send(smsg, message)                                     \
    {                                                                   \
        if (OS_Smsg_Check(smsg))                                        \
            _OS_WAIT_EVENT(OS_Smsg_IsFree(smsg));                       \
        __OS_SMSG_DI();                                                 \
        smsg = (OST_SMSG) (message);                                    \
        __OS_SMSG_RI();                                                 \
    }

//------------------------------------------------------------------------------
// Send simple message. If message already exists then it will be overwritten.

#define OS_Smsg_Send_Now(smsg, message)                                 \
    {                                                                   \
        OS_Flags.bEventError = 0;                                       \
        if (OS_Smsg_Check(smsg)) OS_Flags.bEventError = 1;              \
        __OS_SMSG_DI();                                                 \
        smsg = (OST_SMSG) (message);                                    \
        __OS_SMSG_RI();                                                 \
    }


//------------------------------------------------------------------------------
// Send simple message. If message already exists then wait when it became free.
// Exit if timeout expired.

#define OS_Smsg_Send_TO(smsg, message, timeout)                     \
    {                                                               \
        OS_Flags.bTimeout = 0;                                      \
        if (OS_Smsg_Check(smsg))                                    \
        {                                                           \
            _OS_WAIT_EVENT_TIMEOUT(OS_Smsg_IsFree(smsg), timeout);  \
        }                                                           \
        if (!OS_IsTimeout()) {                                      \
            __OS_SMSG_DI();                                         \
            smsg = (OST_SMSG) (message);                            \
            __OS_SMSG_RI();                                         \
        }                                                           \
    }


//------------------------------------------------------------------------------
// Wait for simple message. After accepting simple message is cleared.

#define OS_Smsg_Wait(smsg, os_smsg_type_var)                        \
    {                                                               \
        _OS_WAIT_EVENT(OS_Smsg_Check(smsg));                        \
        OS_Smsg_Accept(smsg, os_smsg_type_var);                     \
    }


//------------------------------------------------------------------------------
// Wait for simple message. After accepting simple message is cleared.
// Exit if timeout expired.

#define OS_Smsg_Wait_TO(smsg, os_smsg_type_var, timeout)            \
    {                                                               \
        _OS_WAIT_EVENT_TIMEOUT(OS_Smsg_Check(smsg), timeout);       \
        if (!OS_IsTimeout()) {                                      \
            OS_Smsg_Accept(smsg, os_smsg_type_var);                 \
        }                                                           \
    }

//------------------------------------------------------------------------------
// Accept existing simple message. After accepting message is cleared.

#define OS_Smsg_Accept(smsg, os_smsg_type_var)                      \
    {                                                               \
        __OS_SMSG_DI();                                             \
        os_smsg_type_var = smsg;                                    \
        _OS_Smsg_MakeFree(smsg);                                    \
        __OS_SMSG_RI();                                             \
    }





//------------------------------------------------------------------------------
#ifdef OS_ENABLE_INT_SMSG
//------------------------------------------------------------------------------

// Send simple message from interrupt. If message already exists then it will be overwritten.
#define OS_Smsg_Send_I(smsg, message)                               \
    {                                                               \
        OS_Flags.bEventError = 0;                                   \
        if (OS_Smsg_Check(smsg)) OS_Flags.bEventError = 1;          \
        smsg = (OST_SMSG) (message);                                \
    }


//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------


//******************************************************************************
//  Alternate names
//******************************************************************************

#if !defined(__OSACCS__) && !defined(__OSAMIKROC__)

#define OS_SMsg_Create          OS_Smsg_Create     
#define OS_SMsg_Check           OS_Smsg_Check   
#define OS_SMsg_Accept          OS_Smsg_Accept  
#define OS_SMsg_Send            OS_Smsg_Send    
#define OS_SMsg_Send_TO         OS_Smsg_Send_TO 
#define OS_SMsg_Send_Now        OS_Smsg_Send_Now
#define OS_SMsg_Send_I          OS_Smsg_Send_I  
#define OS_SMsg_Wait            OS_Smsg_Wait    
#define OS_SMsg_Wait_TO         OS_Smsg_Wait_TO 
#define OS_SMsg_IsFree          OS_Smsg_IsFree


#endif












